# Generated by Django 2.2.28 on 2022-07-14 06:50

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('smev_agent_client', '0001_initial'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='entry',
            index=models.Index(fields=['date_time'], name='smev_agent_client_date_ti_e7c85f_idx'),
        ),
    ]
