# Generated by Django 2.2.28 on 2022-07-08 07:36

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('request', models.TextField(verbose_name='Запрос')),
                ('response', models.TextField(blank=True, null=True, verbose_name='Ответ на запрос')),
                ('error', models.TextField(blank=True, null=True, verbose_name='Ошибка')),
                ('date_time', models.DateTimeField(verbose_name='Дата и время отправки запроса')),
            ],
            options={
                'verbose_name': 'Журнал запросов',
                'verbose_name_plural': 'Журнал запросов',
            },
        ),
    ]
